/* -*- c++ -*- */
/*
 * Copyright 2008,2009,2013 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_GR_SENNH_BAT_SINK_IMPL_B_H
#define INCLUDED_GR_SENNH_BAT_SINK_IMPL_B_H

#define NEW_BLOCK_LEN  0.8
#define ZERO_LEN       0.35
#define ONE_LEN        0.21

#define MIN_TOLERANCE  0.9
#define MAX_TOLERANCE  1.1

#define MIN_LEN(l,s)  ((unsigned)((float)s * l * MIN_TOLERANCE + 0.5) - 1)
#define MAX_LEN(l,s)  ((unsigned)((float)s * l * MAX_TOLERANCE + 0.5) + 1)

#include "sennh_bat_sink_b.h"

namespace gr {
  namespace blocks {

    class sennh_bat_sink_b_impl : public sennh_bat_sink_b
    {
    private:
      unsigned d_sample_rate;
      unsigned int d_curtelegram;
      unsigned int d_lasttelegram;
      int d_tpos;
      int d_ncnt;
      bool d_havetelegram;

      void (*d_cbFunc)(void*, unsigned int);
      void *d_cbParam;

      unsigned d_newblock_cnt;
      unsigned d_zero_cnt_min;
      unsigned d_zero_cnt_max;
      unsigned d_one_cnt_min;
      unsigned d_one_cnt_max;

      boost::mutex d_mutex;

    protected:
      bool stop();

    public:
      sennh_bat_sink_b_impl(unsigned int sample_rate);
      ~sennh_bat_sink_b_impl();

      
      bool gettelegram(unsigned int *telegram);
      void setcallback(void (*cbFunc)(void*, unsigned int), void *cbParam);
      void reset();

      int work(int noutput_items,
	       gr_vector_const_void_star &input_items,
	       gr_vector_void_star &output_items);
    };

  } /* namespace blocks */
} /* namespace gr */

#endif /* INCLUDED_GR_SENNH_BAT_SINK_B_IMPL_H */
